#!/usr/bin/env python
# -*- coding: UTF-8 -*-
import logging
from os import listdir
from os.path import abspath, basename, isdir, isfile, join, splitext
from pyunpack import Archive, PatoolError
from subprocess import check_output


__all__ = ["decompress"]
logger = logging.getLogger("main")

ARCHIVE_EXCL = lambda f: basename(f) in ["README", "README.md"] or \
                         f.endswith(".txt")


def decompress(filename, temp_dir):
    """
    Attempts to decompress the input file to a temporary folder. If Patool fails
     to unpack it, it is assumed that the file is not an archive.

    :param filename: path to the input file
    :param temp_dir: temporary directory for decompressed files
    :return:         list of files (these extracted if decompression was
                      performed or the input filename otherwise)
    """
    # set the temporary folder name
    base, ext = splitext(basename(filename))
    base, ext2 = splitext(base)
    if ext2 != '':
        ext = ext2
    tmp_dir = join(abspath(temp_dir), base)
    # try to list files from the archive (do not re-decompress if files are
    #  already present)
    if isdir(tmp_dir) and len(listdir(tmp_dir)) > 0:
        logger.info("Listing files from '{}'...".format(filename))
        try:
            out = check_output(["patool", "list", filename])
            files, bad = [], False
            for line in out.split('\n'):
                if line.startswith("patool: "):
                    break
                fn = join(tmp_dir, line)
                if not isfile(fn) and not ARCHIVE_EXCL(fn):
                    bad = True
                    break
                if not ARCHIVE_EXCL(fn):
                    files.append(fn)
            if not bad:
                # if all required files are already decompressed, just return
                #  the list of file paths
                return True, files
        except CalledProcessError:
            logger.debug("Not an archive, continuing...")
            return False, [filename]
    # now extract files
    logger.info("Decompressing '{}' (if archive)...".format(filename))
    archive = Archive(filename)
    try:
        archive.extractall(tmp_dir, auto_create_dir=True)
    except (PatoolError, ValueError) as e:
        if str(e).startswith("patool can not unpack"):
            logger.debug("Not an archive, continuing...")
            return False, [filename]
        else:
            logger.exception(e)
    # retrieve the list of extracted files
    return True, [join(tmp_dir, fn) for fn in listdir(tmp_dir) \
                  if not ARCHIVE_EXCL(fn)]



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
import logging
import os
import re
import shutil
import string
from collections import deque
from os.path import exists, isdir, isfile, join, relpath, splitext
from subprocess import Popen, PIPE


__all__ = ["DumperTemplate"]
logger = logging.getLogger("main")


class DumperTemplate(object):
    """
    Dumper template class for handling base operations on the memory dump for
     handled applications. It can execute Volatility commands using the attached
     VolatilityMemDump instance and carve files using Foremost.
    """
    _predef_messages = [
        "Raw data (.data files) requires manual handling ;\n"
        "Follow this procedure:\n"
        " 1. Open the collected resources with Gimp\n"
        " 2. Set the width and height to the expected screen resolution\n"
        " 3. Set another color palette than 'RVB'\n"
        "Restart this procedure by setting other parameters for width|height"
        "|palette.",
        "VAD objects (collected with 'vaddump' require further research ;\n"
        "If you know some keywords that could match text elements, you can grep"
        " the .dmp objects.",
    ]
    messages = []
    procnames = None  # by default, the GenericDumper does not correspond to
                      # any particular process name, meaning that a dumper
                      # inheriting this class without redefining procnames will
                      # be processed regardless of the process list

    def __init__(self, dump, name, pids):
        """
        Simple constructor for setting base attributes.

        :param dump: VolatilityMemDump instance
        :param name: application name
        :param pids: list of pids corresponding to the application name in dump
        """
        assert isinstance(name, str)
        assert isinstance(pids, list) and all(x.isdigit() for x in pids)
        self.dump = dump
        self.pids = pids
        self._resname = lambda p, c, f=None: (name.lower() + "{}-{}{}") \
                                .format(["-{}".format(p), ""][p is None], c, \
                                        [".{}".format(f), ""][f is None])
        self.__nopid = []

    def _dump_dir(self, *subdirs):
        """
        Compose the dump directory path and create it if it does not exist.

        :param *subdirs: string pieces of the dump directory path
        :return: the dump directory path as a single string
        """
        assert all(isinstance(x, str) for x in subdirs)
        d = join(self.dump.out_dir, *subdirs)
        if not isdir(d):
            os.makedirs(d)
        return d

    def _dump_file(self, content, cmd, fmt="txt", verbose=True):
        """
        Save a resource to a destination respecting naming conventions.

        :param res: resource file to be saved
        :param pid: PID from which the resource was extracted
        :param cmd: Volatility command used to extract the resource
        :param fmt: extension of the destination resource
        """
        content = content or ""
        assert isinstance(content, str)
        assert isinstance(cmd, str)
        assert isinstance(fmt, str)
        assert isinstance(verbose, bool)
        if content.strip() == "":
            logger.debug("Empty content, no resource saved")
            return
        d = join(self.dump.out_dir,
                 self._resname(self.dump.config.PID, cmd, fmt))
        with open(d, 'wb') as f:
            f.write(content)
        if verbose:
            logger.info("> {}".format(relpath(d)))
        return d

    def _memdump(self, verbose=True):
        """
        Executes the 'memdump' Volatility command for extracting the related
         process memory.

        :param verbose: display the log message after saving the memory dump
        :return: path to the process memory dump
        """
        assert isinstance(verbose, bool)
        cmd, pid = 'memdump', self.dump.config.PID
        out = self.dump.call(cmd,
            "-p {} --dump-dir {}".format(pid, self.dump.out_dir))
        src = join(self.dump.out_dir, "{}.dmp".format(pid))
        dst = join(self.dump.out_dir, self._resname(pid, cmd, "data"))
        shutil.move(src, dst)
        if verbose:
            logger.info("> {}".format(dst))
        return dst

    def _memsearch(self, split_on_nullbyte=False):
        """
        Executes the 'memdump' Volatility command then parse the collected dump
         against used-defined regular expressions.
        Valid patterns structure:
          self.re_patterns := list(tuples(re_pattern, format, short_descr))
        
        :param split_on_nullbyte: split the matched string on nullbyte and take
                                   only the first part
        """
        if not hasattr(self, "re_patterns"):
            logger.warning("No memory search performed (no pattern found)")
            return
        dump = self._memdump(verbose=False)
        with open(dump) as f:
            content = f.read()
        for pattern, fmt, descr in self.re_patterns:
            r = re.compile(pattern, re.M + re.S)
            out = r.search(content)
            if out is not None:
                out = out.group()
                #FIXME: split on nullbyte for each occurrence then join all
                if split_on_nullbyte:
                    out = out.split('\x00', 1)[0]
                self._dump_file(out, 'memdump-{}'.format(descr), fmt)
        os.remove(dump)

    def _run(self):
        """
        Consecutively execute the public 'run()' method on each PID from the
         pids list.
        """
        if len(self.pids) == 0:
            logger.debug("Dumping information...")
            self.run()
        else:
            for pid in self.pids:
                logger.debug("Dumping for PID {}...".format(pid))
                self.dump.config.PID = pid
                self.run()
        return self.messages

    def _vaddump(self, verbose=True):
        """
        Executes the 'vaddump' Volatility command for extracting the VAD tree
         for a given process.

        :param verbose: display the log message after grabbing the VAD nodes
        :return: path to the VAD dump directory
        """
        assert isinstance(verbose, bool)
        dump_dir = self._dump_dir('vad')
        out = self.dump.call('vaddump', "-p {} -D {}" \
                             .format(self.dump.config.PID, dump_dir))
        if verbose:
            logger.info("> {}".format(dump_dir))
        return dump_dir

    def _vadsearch(self, stop=True, include_pattern=False, reduce_text=False):
        """
        Executes the 'vaddump' Volatility command then parse the collected VAD
         nodes against used-defined patterns for collecting resources.
        Valid patterns structure:
          self.fmt_patterns := list(tuples(start_pattern, end_pattern, format))
        
        :param stop: stop after the first resource matching the patterns
        :param include_pattern: include the found pattern when saving resource
        :param reduce_text: use GenericDumper to reduce the output (makes sense
                            if the output is text)
        """
        assert isinstance(stop, bool)
        assert isinstance(include_pattern, bool)
        assert isinstance(reduce_text, bool)
        if not hasattr(self, "fmt_patterns"):
            logger.warning("No VAD search performed (no pattern found)")
            return
        dump_dir = self._vaddump(verbose=False)
        for fn in os.listdir(dump_dir):
            with open(join(dump_dir, fn), 'rb') as f:
                node = f.read()
            # choose the first start matching pattern in the provided list of
            #  format patterns
            found = False
            for start, end, fmt in self.fmt_patterns:
                try:
                    out = ["", start][include_pattern] + node.split(start, 1)[1]
                    out = out.split(end, 1)[0] + ["", end][include_pattern]
                    found = True
                    break
                except:
                    continue
            if found:
                if reduce_text:
                    out = GenericDumper.reduce_text(out)
                self._dump_file(out.rstrip('\x00'), 'vadnode', fmt)
                if stop:
                    break
        shutil.rmtree(dump_dir)

    def _yarascan(self, pattern, verbose=True):
        """
        Executes the 'yarascan' Volatility command for matching a pattern from
         the related process memory.
        
        :param pattern: yara pattern
        :param verbose: display the log message after saving the scan result
        """
        assert isinstance(pattern, str)
        assert isinstance(verbose, bool)
        cmd, pid = 'yarascan', self.dump.config.PID
        out = self.dump.call(cmd, "-p {} -Y '/{}/'".format(pid, pattern))
        print(out)
        if len(out.strip()) == 0:
            return
        i = 0
        dst = join(self.dump.out_dir,
                   self._resname(pid, cmd + "-{:0<2}".format(i), "txt"))
        while exists(dst):
            i += 1
            dst = join(self.dump.out_dir,
                       self._resname(pid, cmd + "-{:0<2}".format(i), "txt"))
        with open(dst, 'wb') as f:
            f.write(out)
        if verbose:
            logger.info("> {}".format(dst))

    def carve(self, filepath, types=(), clean=False):
        """
        Carve files of given types from the input resource with Foremost.

        :param filepath: file path of the resource to be carved
        :param types: list of extensions to be carved from the resource
        """
        assert isfile(filepath)
        assert (isinstance(types, tuple) or isinstance(types, list)) and \
               all(isinstance(x, str) for x in types)
        assert isinstance(clean, bool)
        logger.debug("> Carving with Foremost...")
        folder, _ = splitext(filepath)
        opt = ["", "-t {}".format(",".join(types))][len(types) > 0]
        cmd = "foremost {} -o {} {}".format(filepath, folder, opt)
        logger.debug(">> {}".format(cmd))
        p = Popen(cmd.split(), stdin=PIPE, stdout=PIPE, stderr=PIPE)
        out, err = p.communicate()
        with open(join(folder, "audit.txt")) as f:
            disp = False
            for line in f.readlines():
                line = line.strip()
                if "Finish:" in line:
                    break
                if all([x in line for x in ("Num", "Name", "Size", "Comment")]):
                    disp = True
                if disp and len(line) > 0 and ":" in line:
                    fn = line.split(":", 1)[1].strip().split()[0]
                    _, ext = splitext(fn)
                    fp = join(folder, ext[1:], fn)
                    logger.info("> {}".format(fp))
        if clean:
            os.remove(filepath)

    def run(self):
        """
        Executes the 'memdump' Volatility command for extracting the related
         process memory.
        Public run method to be overridden for the operations to be applied to
         the related application.

        :param verbose: display the log message after saving the memory dump
        """
        raise NotImplementedError("Subclass GenericDumper and override run()")

    @staticmethod
    def reduce_text(text, alphabet=string.printable, wsize=5, threshold=3):
        """
        Determines start|end bounds based on a window of booleans telling if the
         characters are well in the allowed alphabet with the constraints that
         the first character in the window must be a not-allowed one and the
         number of not-allowed ones in the window is above the threshold
        """
        assert isinstance(text, str)
        assert isinstance(alphabet, str)
        assert isinstance(wsize, int)
        assert isinstance(threshold, int)
        # NB: the text is in UTF-16 little-endian and can simply be retrieved
        #     by removing the nullbytes after each normal character
        text = text.replace('\x00', '').replace('\r\n', '\n')
        halflen = len(text) / 2
        bounds = [halflen, len(text) - halflen - 1]
        for i, t in enumerate([text[:halflen][::-1], text[halflen:]]):
            w = deque(maxlen=wsize)
            bound_adapted = False
            # slide along the text to adapt the bound
            for k, c in enumerate(t):
                w.append(c not in alphabet)
                if w[0] and sum(w) >= threshold:
                    bounds[i] = k - len(w) + 1
                    bound_adapted = True
                    break
            # handle the window of the text tail if bound not adapted yet
            if not bound_adapted:
                for j in range(wsize - 1):
                    new_th = int(round(float(threshold) / wsize * len(w)))
                    if w.popleft() and sum(w) >= new_th:
                        bounds[i] = k - len(w)
                        break
        return text[halflen-bounds[0]:halflen+bounds[1]]



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
import re
from .template import DumperTemplate


__all__ = [
    "Clipboard",
    "CriticalProcessesInfo",
    "DumpInfo",
    "Mimikatz",
    "UserHashes",
]


class Clipboard(DumperTemplate):
    """
    Dumper for collecting the content of the clipboard.
    """
    def run(self):
        out = self.dump.call("clipboard", failmode="warn")
        if len(out.split('\n')) > 3:
            self._dump_file(out, "content")
        else:
            logger.debug("Empty content, no resource saved")


class CriticalProcessesInfo(DumperTemplate):
    """
    Dumper for checking critical processes.
    """
    def run(self):
        self.dump.config.PID = None
        kw = {'options': "--output=greptext", 'failmode': "silent"}
        pslist, pstree, psscan, psxview = \
            [self._dump_file(self.dump.call(cmd, **kw), cmd) \
             for cmd in ['pslist', 'pstree', 'psscan', 'psxview']]
        info = ""
        # check for single instance of lsass.exe and services.exe
        for ps in ["lsass.exe", "services.exe"]:
            instances = []
            with open(psscan) as f:
                for l in f:
                    l = l.split("|")
                    if ps in l:
                        instances += [(l[3], l[4])]  # PID, PPID
            pids = list(map(lambda x: x[0], instances))
            with open(psxview) as f:
                for l in f:
                    l = l.split("|")
                    pid = l[3]
                    if ps in l and pid not in pids:
                        instances += [(pid, "N/A")]  # PID, PPID
            if len(instances) > 1:
                info += "Multiple instances of {} found".format(ps)
                for pid, ppid in sorted(instances, key=lambda x: int(x[0])):
                    info += "\n{} (parent: {})".format(pid, ppid)
                self._dump_file(info, "multiple-{}".format(ps.split(".", 1)[0]))


class DumpInfo(DumperTemplate):
    """
    Dumper of general information about the memory image.
    """
    def run(self):
        """
        Executes a series of informational Volatility commands.
        """
        for cmd in ['crashinfo', 'hibinfo', 'vboxinfo', 'vmwareinfo']:
            self._dump_file(self.dump.call(cmd, failmode="silent"), cmd)


class Mimikatz(DumperTemplate):
    """
    Dumper for collecting user passwords present in memory.
    """
    def run(self):
        cmd = "mimikatz"
        self._dump_file(self.dump.call(cmd, failmode="warn"), cmd)


class UserHashes(DumperTemplate):
    """
    Dumper of Windows user hashes from the memory image.
    """
    def run(self):
        """
        Executes 'hivelist' and 'hashdump' Volatility commands in order to
         dump user hashes.
        """
        out = self.dump.call("hivelist")
        re1, re2 = re.compile(r'(sam)$', re.I), re.compile(r'(system)$', re.I)
        sam, system = None, None
        for line in out.split('\n'):
            if line.startswith("0x"):
                start, end, hive = line.split(" ", 2)
                if re1.search(hive) is not None:
                    sam = start
                if re2.search(hive) is not None:
                    system = start
        cmd = "hashdump"
        if sam is not None and system is not None:
            out = self.dump.call(cmd, "{} -s {}".format(sam, system))
            self._dump_file(out, cmd)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
from os.path import abspath, dirname, join
from setuptools import setup, find_packages
try: # for pip >= 10
    from pip._internal.req import parse_requirements
except ImportError: # for pip <= 9.0.3
    from pip.req import parse_requirements


currdir = abspath(dirname(__file__))
with open(join(currdir, 'README.md')) as f:
    long_descr = f.read()

requirements = parse_requirements("requirements.txt", session=False)
setup(
  name = "appmemdumper",
  packages = find_packages(),
  author = "Alexandre D\'Hondt",
  author_email = "alexandre.dhondt@gmail.com",
  url = "https://github.com/dhondta/AppmemDumper",
  license = "AGPLv3",
  version = "2.1.3",
  description = "This tool allows to collect various forensics artifacts in "
                "Windows memory dumps using Volatility and Foremost for common "
                "Windows applications.",
  long_description=long_descr,
  long_description_content_type='text/markdown',
  scripts = ["app-mem-dumper"],
  keywords = ["forensics", "volatility", "foremost", "artifacts", "windows", "applications"],
  classifiers = [
    'Development Status :: 4 - Beta',
    'Environment :: Console',
    'Intended Audience :: Developers',
    'Intended Audience :: Information Technology',
    'Intended Audience :: End Users/Desktop',
    'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.7',
  ],
  install_requires=[str(r.req) for r in requirements],
  python_requires = '>=2.7,<3',
)



#!/usr/bin/python
# -*- coding: UTF-8 -*-
import logging
import os
import shlex
import shutil
import StringIO
import sys
import volatility.addrspace as addrspace
import volatility.conf as conf
import volatility.commands as commands
import volatility.constants as constants
import volatility.debug as debug
import volatility.obj as obj
import volatility.registry as registry
import volatility.utils as utils
from copy import deepcopy
from os.path import abspath, dirname, exists, isdir, isfile
from subprocess import check_output, CalledProcessError, PIPE, Popen
from .dumpers import *

try:
    Popen(["foremost", "-h"], stdout=PIPE, stderr=PIPE)
except OSError:
    print("Missing dependencies, please run 'sudo apt-get install foremost'")
    sys.exit(1)


__all__ = ["DUMPERS", "VolatilityMemDump"]
CMD = 'volatility {opt} {cmd}'


is_memdump = lambda x: isinstance(x, VolatilityMemDump)
logger = logging.getLogger("main")


class VolatilityAppDumper(object):
    """
    Volatility application dumper "relay" class for choosing the right
     application dumper based on the process name. It is called by
     VolatilityMemDump at process list creation to attach the desired dumpers.
     It instantiates a child class of GenericDumper adapted to the given
     application name.
    """
    def __init__(self, dump, pname=None):
        """
        Attaches the parent memory dump file and instantiates the right child
         class of GenericDumper adapted for the input application name.

        :param dump: VolatilityMemDump instance
        :param pname: application process name
        """
        assert isinstance(dump, VolatilityMemDump)
        assert pname is None or isinstance(pname, str)
        # first, find if PID's exist when a process name is given
        pids = []
        if pname is not None:
            for p in dump.processes:
                if pname == str(p.ImageFileName).lower():
                    pids.append(str(p.UniqueProcessId))
            if len(pids) == 0:
                logger.error("No PID found for '{}'".format(pname))
                return
        # second, find the right application dumper for the given process name
        #  and attach the dumper to the memory dump if a dumper is found and if
        #  the related application is handled
        # if no process name was given and there exists a general-purpose dumper
        #  (that has no list of process names), create a dumper
        for cls in DUMPERS:
            pnames = globals()[cls].procnames
            if pnames is not None:
                pnames = [p.lower() for p in pnames]
            if (pname is None and pnames is None) or \
               (pname is not None and pnames is not None and pname in pnames):
                if cls in dump._selected_apps:
                    if pname is not None:
                        dump._stats[0].append(pname)  # found
                    name = pname.split('.', 1)[0] if pname is not None else cls
                    self.dumper = globals()[cls](dump, name, pids)
                    dump.apps[cls] = self.dumper
                else:
                    if pname is not None:
                        dump._stats[1].append(pname)  # not selected
                if pname is not None and pnames is not None:
                    return
        if pname is not None:
            dump._stats[2].append(pname)              # not handled


class VolatilityMemDump(object):
    """
    Volatility memory dump class that handles profile search and processes list
     retrieval. It allows either to execute a command from the Python API or to
     call Volatility in a subprocess.
    At processes list creation, it creates a list of Volatility application
     dumpers for the common applications handled by this tool by instantiating
     VolatilityAppDumper. It also provides a 'dump()' method to trigger the
     dumping of resources with the application dumpers.
    """
    class OutputParser(dict):
        """
        Dictionary that parses the output of a Volatility command at creation with:
         - every colon-separated parameter as a (key, value) pair
         - other lines under the 'others' key
        """
        def __init__(self, output):
            """
            Dictionary constructor that parses the Volatility command output.

            :param output: Volatility command output as a string
            """
            assert isinstance(output, str)
            self['others'] = []
            for line in output.split('\n'):
                line = line.strip().split(':', 1)
                if len(line) == 1:
                    if line[0].strip() != '':
                        self['others'].append(line[0].strip())
                else:
                    self[line[0].strip()] = line[1].strip()
    parsers = {
        'imageinfo': lambda o: VolatilityMemDump.OutputParser(o) \
                               ["Suggested Profile(s)"].split(", "),
    }

    def __init__(self, dump, selected_apps, out_dir="files", plugins_dir=None,
                 from_cache=True):
        """
        Determines the profile, retrieves the list of processes and creates the
         list of application dumpers.

        :param dump: Volatility memory dump filename
        :param selected_apps: list of the application names to be handled
        :param out_dir: output directory for retrieved resources
        :param plugins_dir: Volatility custom plugins directory
        :param from_cache: boolean indicating if the profile must be retrieved
                           from a cache file in /tmp or found by Volatility
        """
        short = dump
        dump = abspath(dump)
        assert isfile(dump), "{} is not a dump file".format(dump)
        assert all(x in DUMPERS for x in selected_apps), "Unknown dumper(s)"
        assert plugins_dir is None or isdir(plugins_dir), "Bad plugins dir"
        assert isinstance(from_cache, bool)
        self._selected_apps = selected_apps
        logger.debug("Setting output directory to '{}'...".format(out_dir))
        self.out_dir = abspath(out_dir)
        # initialize dump opening
        registry.PluginImporter()
        self.__is_profile_tested = False
        self.config = conf.ConfObject()
        if plugins_dir is not None:
            logger.debug("Setting plugins directory to '{}'..."
                         .format(plugins_dir))
            self.config.plugins = abspath(plugins_dir)
        for cls in [commands.Command, addrspace.BaseAddressSpace]:
            registry.register_global_options(self.config, cls)
        self.__commands = {k.lower(): v for k, v in \
                          registry.get_plugin_classes(commands.Command).items()}
        self.config.LOCATION = "file://{}".format(dump)
        # get the right dump profile and test it while getting processes
        logger.info("Opening dump file '{}'...".format(short))
        self.__is_profile_tested = self.__get_profile(from_cache)

    def __get_profile(self, from_cache=True):
        """
        Get the dump image profile by loading it from a cache file or by running
         'imageinfo' Volatility command.

        :param from_cache: load the profile from the cache file if it exists
        :return: boolean indicating if the profile is tested as correct
        """
        assert isinstance(from_cache, bool)
        logger.info("Getting profile...")
        # get available profiles and commands
        available_profiles = registry.get_plugin_classes(obj.Profile)
        compatible_profiles = []
        # load profile from cache or use the 'imageinfo' command to find it
        tmp = "/tmp/appmemdumper/profile"
        if from_cache:
            if isfile(tmp):
                with open(tmp) as f:
                    self.config.PROFILE = f.read().strip()
                if not self.__get_pslist():
                    self.config.PROFILE = None
                else:
                    compatible_profiles = [self.config.PROFILE]
            else:
                from_cache = False
        if not from_cache or self.config.PROFILE is None:
            self.config.PROFILE = "WinXPSP2x86"  # reset to default
            compatible_profiles = [p for p in self.call('imageinfo',
                                   parser=self.parsers['imageinfo'])]
        self.__profiles = {p: c for p, c in available_profiles.items() \
                           if p in compatible_profiles}
        # try 'pslist' command on the found profile ; if an error occurs, try
        #  the next profile in the list
        cursor, is_profile_tested = 0, False
        while not is_profile_tested and cursor < len(self.__profiles):
            self.config.PROFILE = self.__profiles.keys()[cursor]
            logger.debug("Profile: {}".format(self.config.PROFILE))
            is_profile_tested = self.__get_pslist()
            cursor += 1
        if not is_profile_tested:
            logger.error("No suitable profile could be found ; please check "
                         "that your memory dump is supported by Volatility")
            sys.exit(2)
        # create a cache file with the profile name
        if not isdir(dirname(tmp)):
            os.makedirs(dirname(tmp))
        with open(tmp, 'w') as f:
            f.write(self.config.PROFILE)
        logger.info("> Selected profile: {}".format(self.config.PROFILE))
        return is_profile_tested

    def __get_pslist(self):
        """
        Run 'pslist' Volatility command to check if the profile is correctly
         set, compute the list of processes and attach dumper objects for known
         applications.

        :return: boolean telling if the profile is correct
        """
        if hasattr(self, "processes"):
            return True
        logger.info("Getting processes...")
        try:
            self.processes = self.execute('pslist')
        except:
            logger.debug("Wrong profile, re-getting profile...")
            return False
        # parse the list of processes and create objects for each set of
        #  application instances that are handled by the present tool
        self.apps = {}
        self._stats = [[], [], []]
        procnames = set([str(p.ImageFileName) for p in self.processes])
        # first, search for general-purpose dumpers
        VolatilityAppDumper(self)
        # second, search for specific-purpose dumpers per process name
        for n in sorted(procnames, key=lambda n: n.lower()):
            VolatilityAppDumper(self, n.lower())
        for i, t in enumerate(["Found", "Not selected", "Not handled"]):
            s = self._stats[i]
            if len(s) > 0:
                logger.debug("{}: {}".format(t.ljust(12), ", ".join(s)))
        if len(self.apps) == 0:
            logger.warning("No application to be handled")
        return True

    def call(self, command, options=None, parser=None, failmode="error"):
        """
        Run Volatility with given command and options using subprocess.

        :param command: Volatility command name
        :param options: valid command-related options
        :param parser: None or function for parsing the output
        :param failmode: string indicating the fail mode (error|warn|silent)
        :return: parsed output
        """
        assert isinstance(command, str)
        assert options is None or isinstance(options, str)
        assert parser is None or callable(parser)
        assert failmode in ["error", "warn", "silent"]
        # compose the list of options with already known and input information
        options = [options] if options is not None else []
        if self.__is_profile_tested:
            options.insert(0, "--profile={}".format(self.config.PROFILE))
        filepath = self.config.LOCATION.split("://", 1)[1]
        options.insert(0, "--file=\"{}\"".format(filepath))
        if self.config.plugins is not None and len(self.config.plugins) > 0:
            options.insert(0, "--plugins={}".format(self.config.plugins))
        cmd = CMD.format(cmd=command, opt=" ".join(options))
        # run the resulting command with a subprocess
        logger.debug("> Calling command '{}'...".format(command))
        logger.debug(">> {}".format(cmd))
        p = Popen(shlex.split(cmd), stdin=PIPE, stdout=PIPE, stderr=PIPE)
        out, err = p.communicate()
        if "ERROR" in err:
            if failmode == "silent":
                return
            if "You must specify something to do (try -h)" in err:
                msg = "Plugin does not exist"
            else:
                msg = "({}) {}".format(*map(lambda x: x.strip(),
                                            err.split(":", 2)[1:3]))
            if failmode == "warn":
                logger.warning(msg)
                return
            elif failmode == "error":
                logger.error(msg)
                raise CalledProcessError(1, cmd)
        out = out.replace('\r\n', '\n')
        # then parse the output with the given parser and return the result
        if parser is not None:
            out = parser(out)
        return out

    def dump(self):
        """
        Dump resources for every known application in the memory dump.
        """
        if exists(self.out_dir):
            shutil.rmtree(self.out_dir) if isdir(self.out_dir) \
                else os.remove(self.out_dir)
        os.makedirs(self.out_dir)
        # run each dumper and collect information messages for guidance
        msgs = {}
        for n, a in self.apps.items():
            logger.info("Processing dumper '{}'...".format(n))
            for msg in a._run():
                msgs.setdefault(msg, [])
                msgs[msg].append(n)
        # finally, warn the collected messages
        for m, n in msgs.items():
            logger.warning("\nThe following applies to collected objects of:\n-"
                           " {}\n\n{}\n".format("\n- ".join(n), m))
            
    def execute(self, command, nopid=False, text=False, parser=None):
        """
        Run Volatility command through the dedicated Python API.

        :param command: Volatility command name
        :param nopid: ensure no PID in the config object (for some commands that
                      search for application artifacts without requiring a PID)
        :param text: convert the result into text
        :param parser: None or function for parsing the output
        :return: parsed output
        """
        assert command in self.__commands.keys()
        assert isinstance(nopid, bool)
        assert isinstance(text, bool)
        assert parser is None or callable(parser)
        logger.debug("> Executing command '{}'...".format(command))
        if nopid:
            pid = self.config.PID
            self.config.PID = None
        # run the command with the API
        command = self.__commands[command](deepcopy(self.config))
        result = command.calculate()
        # convert the result to text if required
        if text:
            io = StringIO.StringIO()
            command.render_text(io, result)
            out = io.getvalue().replace('\r\n', '\n')
        else:
            out = list(result)
        if nopid:
            self.config.PID = pid
        # then parse the output with the given parser and return the result
        if parser:
            out = parser(out)
        return out
